# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "剩余 %d 分钟去阅读",
    "(active)": "(活跃)",
    "Also available in:": "也可用于：",
    "Archive": "文章归档",
    "Atom feed": "Atom源",
    "Authors": "作者",
    "Categories": "分类",
    "Comments": "评论",
    "LANGUAGE": "简体中文",
    "Languages:": "语言：",
    "More posts about %s": "更多关于文章 %s",
    "Newer posts": "较新的文章",
    "Next post": "下一篇文章",
    "Next": "项下",
    "No posts found.": "没有找到文章",
    "Nothing found.": "没有找到。",
    "Older posts": "以前的文章",
    "Original site": "原文地址",
    "Posted:": "发表于：",
    "Posts about %s": "关于文章 %s",
    "Posts by %s": "文章有%s发布",
    "Posts for year %s": "%s年文章",
    "Posts for {month_day_year}": "{month_day_year}文章",
    "Posts for {month_year}": "{month_year}文章",
    "Previous post": "上一篇文章",
    "Previous": "以前",
    "Publication date": "发布日期",
    "RSS feed": "RSS 源",
    "Read in English": "阅读简体中文",
    "Read more": "阅读更多",
    "Skip to main content": "跳到主内容",
    "Source": "资源",
    "Subcategories:": "子类别：",
    "Tags and Categories": "标签和分类",
    "Tags": "标签",
    "Toggle navigation": "展开导航栏",
    "Uncategorized": "未分类",
    "Up": "向上",
    "Updates": "更新",
    "Write your page here.": "在这里书写你的页面。",
    "Write your post here.": "在这里书写你的文章。",
    "old posts, page %d": "旧文章页 %d",
    "page %d": "第 %d 页",
    "updated": "更新",
}



# -*- coding: utf-8 -*-

# Copyright © 2012-2018 Roberto Alsina and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""The main function of Nikola."""

import os
import shutil
import sys
import traceback
from collections import defaultdict

from blinker import signal
from doit.cmd_auto import Auto as DoitAuto
from doit.cmd_base import TaskLoader
from doit.cmd_clean import Clean as DoitClean
from doit.cmd_completion import TabCompletion
from doit.cmd_help import Help as DoitHelp
from doit.cmd_run import Run as DoitRun
from doit.doit_cmd import DoitMain
from doit.loader import generate_tasks
from doit.reporter import ExecutedOnlyReporter
from logbook import NullHandler

from . import __version__
from .nikola import Nikola
from .plugin_categories import Command
from .utils import (LOGGER, STDERR_HANDLER, STRICT_HANDLER,
                    ColorfulStderrHandler, get_root_dir, req_missing,
                    sys_decode)

try:
    import readline  # NOQA
except ImportError:
    pass  # This is only so raw_input/input does nicer things if it's available


import importlib.machinery

config = {}

# DO NOT USE unless you know what you are doing!
_RETURN_DOITNIKOLA = False


def main(args=None):
    """Run Nikola."""
    colorful = False
    if sys.stderr.isatty() and os.name != 'nt' and os.getenv('NIKOLA_MONO') is None and os.getenv('TERM') != 'dumb':
        colorful = True

    ColorfulStderrHandler._colorful = colorful

    if args is None:
        args = sys.argv[1:]

    oargs = args
    args = [sys_decode(arg) for arg in args]

    conf_filename = 'conf.py'
    conf_filename_changed = False
    for index, arg in enumerate(args):
        if arg[:7] == '--conf=':
            del args[index]
            del oargs[index]
            conf_filename = arg[7:]
            conf_filename_changed = True
            break

    quiet = False
    strict = False
    if len(args) > 0 and args[0] == 'build' and '--strict' in args:
        LOGGER.notice('Running in strict mode')
        STRICT_HANDLER.push_application()
        strict = True
    if len(args) > 0 and args[0] == 'build' and '-q' in args or '--quiet' in args:
        NullHandler().push_application()
        quiet = True
    if not quiet and not strict:
        NullHandler().push_application()
        STDERR_HANDLER[0].push_application()

    global config

    original_cwd = os.getcwd()

    # Those commands do not require a `conf.py`.  (Issue #1132)
    # Moreover, actually having one somewhere in the tree can be bad, putting
    # the output of that command (the new site) in an unknown directory that is
    # not the current working directory.  (does not apply to `version`)
    argname = args[0] if len(args) > 0 else None
    if argname and argname not in ['init', 'version'] and not argname.startswith('import_'):
        root = get_root_dir()
        if root:
            os.chdir(root)
        # Help and imports don't require config, but can use one if it exists
        needs_config_file = (argname != 'help') and not argname.startswith('import_')
    else:
        needs_config_file = False

    sys.path.append('')
    try:
        loader = importlib.machinery.SourceFileLoader("conf", conf_filename)
        conf = loader.load_module()
        config = conf.__dict__
    except Exception:
        if os.path.exists(conf_filename):
            msg = traceback.format_exc(0)
            LOGGER.error('"{0}" cannot be parsed.\n{1}'.format(conf_filename, msg))
            return 1
        elif needs_config_file and conf_filename_changed:
            LOGGER.error('Cannot find configuration file "{0}".'.format(conf_filename))
            return 1
        config = {}

    if conf_filename_changed:
        LOGGER.info("Using config file '{0}'".format(conf_filename))

    invariant = False

    if len(args) > 0 and args[0] == 'build' and '--invariant' in args:
        try:
            import freezegun
            freeze = freezegun.freeze_time("2038-01-01")
            freeze.start()
            invariant = True
        except ImportError:
            req_missing(['freezegun'], 'perform invariant builds')

    if config:
        if os.path.isdir('plugins') and not os.path.exists('plugins/__init__.py'):
            with open('plugins/__init__.py', 'w') as fh:
                fh.write('# Plugin modules go here.')

    config['__colorful__'] = colorful
    config['__invariant__'] = invariant
    config['__quiet__'] = quiet
    config['__configuration_filename__'] = conf_filename
    config['__cwd__'] = original_cwd
    site = Nikola(**config)
    DN = DoitNikola(site, quiet)
    if _RETURN_DOITNIKOLA:
        return DN
    _ = DN.run(oargs)

    if site.invariant:
        freeze.stop()
    return _


class Help(DoitHelp):
    """Show Nikola usage."""

    @staticmethod
    def print_usage(cmds):
        """Print nikola "usage" (basic help) instructions."""
        # Remove 'run'.  Nikola uses 'build', though we support 'run' for
        # people used to it (eg. doit users).
        # WARNING: 'run' is the vanilla doit command, without support for
        #          --strict, --invariant and --quiet.
        del cmds['run']

        print("Nikola is a tool to create static websites and blogs. For full documentation and more information, please visit https://getnikola.com/\n\n")
        print("Available commands:")
        for cmd_name in sorted(cmds.keys()):
            cmd = cmds[cmd_name]
            print("  nikola {:20s} {}".format(cmd_name, cmd.doc_purpose))
        print("")
        print("  nikola help                 show help / reference")
        print("  nikola help <command>       show command usage")
        print("  nikola help <task-name>     show task usage")


class Build(DoitRun):
    """Expose "run" command as "build" for backwards compatibility."""

    def __init__(self, *args, **kw):
        """Initialize Build."""
        opts = list(self.cmd_options)
        opts.append(
            {
                'name': 'strict',
                'long': 'strict',
                'default': False,
                'type': bool,
                'help': "Fail on things that would normally be warnings.",
            }
        )
        opts.append(
            {
                'name': 'invariant',
                'long': 'invariant',
                'default': False,
                'type': bool,
                'help': "Generate invariant output (for testing only!).",
            }
        )
        opts.append(
            {
                'name': 'quiet',
                'long': 'quiet',
                'short': 'q',
                'default': False,
                'type': bool,
                'help': "Run quietly.",
            }
        )
        self.cmd_options = tuple(opts)
        super(Build, self).__init__(*args, **kw)


class Clean(DoitClean):
    """Clean site, including the cache directory."""

    # The unseemly *a is because this API changed between doit 0.30.1 and 0.31
    def clean_tasks(self, tasks, dryrun, *a):
        """Clean tasks."""
        if not dryrun and config:
            cache_folder = config.get('CACHE_FOLDER', 'cache')
            if os.path.exists(cache_folder):
                shutil.rmtree(cache_folder)
        return super(Clean, self).clean_tasks(tasks, dryrun, *a)


# Nikola has its own "auto" commands that uses livereload.
# Expose original doit "auto" command as "doit_auto".
DoitAuto.name = 'doit_auto'


class NikolaTaskLoader(TaskLoader):
    """Nikola-specific task loader."""

    def __init__(self, nikola, quiet=False):
        """Initialize the loader."""
        self.nikola = nikola
        self.quiet = quiet

    def load_tasks(self, cmd, opt_values, pos_args):
        """Load Nikola tasks."""
        if self.quiet:
            DOIT_CONFIG = {
                'verbosity': 0,
                'reporter': 'zero',
            }
        else:
            DOIT_CONFIG = {
                'reporter': ExecutedOnlyReporter,
                'outfile': sys.stderr,
            }
        DOIT_CONFIG['default_tasks'] = ['render_site', 'post_render']
        DOIT_CONFIG.update(self.nikola._doit_config)
        try:
            tasks = generate_tasks(
                'render_site',
                self.nikola.gen_tasks('render_site', "Task", 'Group of tasks to render the site.'))
            latetasks = generate_tasks(
                'post_render',
                self.nikola.gen_tasks('post_render', "LateTask", 'Group of tasks to be executed after site is rendered.'))
            signal('initialized').send(self.nikola)
        except Exception:
            LOGGER.error('Error loading tasks. An unhandled exception occurred.')
            if self.nikola.debug or self.nikola.show_tracebacks:
                raise
            _print_exception()
            sys.exit(3)
        return tasks + latetasks, DOIT_CONFIG


class DoitNikola(DoitMain):
    """Nikola-specific implementation of DoitMain."""

    # overwite help command
    DOIT_CMDS = list(DoitMain.DOIT_CMDS) + [Help, Build, Clean, DoitAuto]
    TASK_LOADER = NikolaTaskLoader

    def __init__(self, nikola, quiet=False):
        """Initialzie DoitNikola."""
        super(DoitNikola, self).__init__()
        self.nikola = nikola
        nikola.doit = self
        self.task_loader = self.TASK_LOADER(nikola, quiet)

    def get_cmds(self):
        """Get commands."""
        # core doit commands
        cmds = DoitMain.get_cmds(self)
        # load nikola commands
        for name, cmd in self.nikola._commands.items():
            cmds[name] = cmd
        return cmds

    def run(self, cmd_args):
        """Run Nikola."""
        args = self.process_args(cmd_args)
        args = [sys_decode(arg) for arg in args]

        if len(args) == 0:
            cmd_args = ['help']
            args = ['help']

        if '--help' in args or '-h' in args:
            new_cmd_args = ['help'] + cmd_args
            new_args = ['help'] + args

            cmd_args = []
            args = []

            for arg in new_cmd_args:
                if arg not in ('--help', '-h'):
                    cmd_args.append(arg)
            for arg in new_args:
                if arg not in ('--help', '-h'):
                    args.append(arg)

        if args[0] == 'help':
            self.nikola.init_plugins(commands_only=True)
        elif args[0] == 'plugin':
            self.nikola.init_plugins(load_all=True)
        else:
            self.nikola.init_plugins()

        sub_cmds = self.get_cmds()

        if any(arg in ("--version", '-V') for arg in args):
            cmd_args = ['version']
            args = ['version']
        if args[0] not in sub_cmds.keys():
            LOGGER.error("Unknown command {0}".format(args[0]))
            sugg = defaultdict(list)
            sub_filtered = (i for i in sub_cmds.keys() if i != 'run')
            for c in sub_filtered:
                d = levenshtein(c, args[0])
                sugg[d].append(c)
            if sugg.keys():
                best_sugg = sugg[min(sugg.keys())]
                if len(best_sugg) == 1:
                    LOGGER.info('Did you mean "{}"?'.format(best_sugg[0]))
                else:
                    LOGGER.info('Did you mean "{}" or "{}"?'.format('", "'.join(best_sugg[:-1]), best_sugg[-1]))
            return 3

        if not sub_cmds[args[0]] in (Help, TabCompletion) and not isinstance(sub_cmds[args[0]], Command):
            if not self.nikola.configured:
                LOGGER.error("This command needs to run inside an "
                             "existing Nikola site.")
                return 3
        try:
            return super(DoitNikola, self).run(cmd_args)
        except Exception:
            LOGGER.error('An unhandled exception occurred.')
            if self.nikola.debug or self.nikola.show_tracebacks:
                raise
            _print_exception()
            return 1

    @staticmethod
    def print_version():
        """Print Nikola version."""
        print("Nikola v" + __version__)


def levenshtein(s1, s2):
    u"""Calculate the Levenshtein distance of two strings.

    Implementation from Wikibooks:
    https://en.wikibooks.org/w/index.php?title=Algorithm_Implementation/Strings/Levenshtein_distance&oldid=2974448#Python
    Copyright © The Wikibooks contributors (CC BY-SA/fair use citation); edited to match coding style and add an exception.
    """
    if len(s1) < len(s2):
        return levenshtein(s2, s1)

    # len(s1) >= len(s2)
    if len(s2) == 0:
        return len(s1)

    previous_row = range(len(s2) + 1)
    for i, c1 in enumerate(s1):
        current_row = [i + 1]
        for j, c2 in enumerate(s2):
            # j+1 instead of j since previous_row and current_row are one character longer than s2
            insertions = previous_row[j + 1] + 1
            deletions = current_row[j] + 1
            substitutions = previous_row[j] + (c1 != c2)
            current_row.append(min(insertions, deletions, substitutions))
        previous_row = current_row

    return previous_row[-1]


def _print_exception():
    """Print an exception in a friendlier, shorter style."""
    etype, evalue, _ = sys.exc_info()
    LOGGER.error(''.join(traceback.format_exception(etype, evalue, None, limit=0, chain=False)).strip())
    LOGGER.notice("To see more details, run Nikola in debug mode (set environment variable NIKOLA_DEBUG=1) or use NIKOLA_SHOW_TRACEBACKS=1")


if __name__ == "__main__":
    sys.exit(main(sys.argv[1:]))



#!/usr/bin/env python3
import subprocess
import os
import argparse

if not os.path.exists('.pypt/gh-token'):
    print("To use this script, you must create a GitHub token first.")
    print("Get a token here: https://github.com/settings/tokens")
    print("Then, put it in a file named .pypt/gh-token")
    exit(1)

parser = argparse.ArgumentParser(description="GitHub Release helper")
parser.add_argument("FILE", nargs=1, help="Markdown file to use")
parser.add_argument("TAG", nargs=1, help="Tag name (usually vX.Y.Z)")

args = parser.parse_args()

if not args.TAG[0].startswith("v"):
    print("WARNING: tag should start with v")
    i = input("Add `v` to tag? [y/n] ")
    if i.lower().strip().startswith('y'):
        args.TAG[0] = 'v' + args.TAG[0]

BASEDIR = os.getcwd()
REPO = 'getnikola/nikola'

subprocess.call(['.pypt/ghrel', args.FILE[0], BASEDIR, REPO, args.TAG[0]])



# -*- coding: utf-8 -*-

# Copyright © 2012-2018 Roberto Alsina and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""Render the tag pages and feeds."""


from nikola.plugin_categories import Taxonomy
from nikola import utils


class ClassifyTags(Taxonomy):
    """Classify the posts by tags."""

    name = "classify_tags"

    classification_name = "tag"
    overview_page_variable_name = "tags"
    overview_page_items_variable_name = "items"
    more_than_one_classifications_per_post = True
    has_hierarchy = False
    show_list_as_subcategories_list = False
    template_for_classification_overview = "tags.tmpl"
    always_disable_rss = False
    always_disable_atom = False
    apply_to_posts = True
    apply_to_pages = False
    omit_empty_classifications = True
    add_other_languages_variable = True
    path_handler_docstrings = {
        'tag_index': """A link to the tag index.

Example:

link://tag_index => /tags/index.html""",
        'tag': """A link to a tag's page. Takes page number as optional keyword argument.

Example:

link://tag/cats => /tags/cats.html""",
        'tag_atom': """A link to a tag's Atom feed.

Example:

link://tag_atom/cats => /tags/cats.atom""",
        'tag_rss': """A link to a tag's RSS feed.

Example:

link://tag_rss/cats => /tags/cats.xml""",
    }

    def set_site(self, site):
        """Set site, which is a Nikola instance."""
        super(ClassifyTags, self).set_site(site)
        self.show_list_as_index = self.site.config['TAG_PAGES_ARE_INDEXES']
        self.template_for_single_list = "tagindex.tmpl" if self.show_list_as_index else "tag.tmpl"
        self.minimum_post_count_per_classification_in_overview = self.site.config['TAGLIST_MINIMUM_POSTS']
        self.translation_manager = utils.ClassificationTranslationManager()

    def is_enabled(self, lang=None):
        """Return True if this taxonomy is enabled, or False otherwise."""
        return True

    def classify(self, post, lang):
        """Classify the given post for the given language."""
        return post.tags_for_language(lang)

    def get_classification_friendly_name(self, classification, lang, only_last_component=False):
        """Extract a friendly name from the classification."""
        return classification

    def slugify_tag_name(self, name, lang):
        """Slugify a tag name."""
        if self.site.config['SLUG_TAG_PATH']:
            name = utils.slugify(name, lang)
        return name

    def get_overview_path(self, lang, dest_type='page'):
        """Return a path for the list of all classifications."""
        if self.site.config['TAGS_INDEX_PATH'](lang):
            path = self.site.config['TAGS_INDEX_PATH'](lang)
            return [_f for _f in [path] if _f], 'never'
        else:
            return [_f for _f in [self.site.config['TAG_PATH'](lang)] if _f], 'always'

    def get_path(self, classification, lang, dest_type='page'):
        """Return a path for the given classification."""
        return [_f for _f in [
            self.site.config['TAG_PATH'](lang),
            self.slugify_tag_name(classification, lang)] if _f], 'auto'

    def provide_overview_context_and_uptodate(self, lang):
        """Provide data for the context and the uptodate list for the list of all classifiations."""
        kw = {
            "tag_path": self.site.config['TAG_PATH'],
            "tag_pages_are_indexes": self.site.config['TAG_PAGES_ARE_INDEXES'],
            "taglist_minimum_post_count": self.site.config['TAGLIST_MINIMUM_POSTS'],
            "tzinfo": self.site.tzinfo,
            "tag_descriptions": self.site.config['TAG_DESCRIPTIONS'],
            "tag_titles": self.site.config['TAG_TITLES'],
        }
        context = {
            "title": self.site.MESSAGES[lang]["Tags"],
            "description": self.site.MESSAGES[lang]["Tags"],
            "pagekind": ["list", "tags_page"],
        }
        kw.update(context)
        return context, kw

    def provide_context_and_uptodate(self, classification, lang, node=None):
        """Provide data for the context and the uptodate list for the list of the given classifiation."""
        kw = {
            "tag_path": self.site.config['TAG_PATH'],
            "tag_pages_are_indexes": self.site.config['TAG_PAGES_ARE_INDEXES'],
            "taglist_minimum_post_count": self.site.config['TAGLIST_MINIMUM_POSTS'],
            "tzinfo": self.site.tzinfo,
            "tag_descriptions": self.site.config['TAG_DESCRIPTIONS'],
            "tag_titles": self.site.config['TAG_TITLES'],
        }
        context = {
            "title": self.site.config['TAG_TITLES'].get(lang, {}).get(classification, self.site.MESSAGES[lang]["Posts about %s"] % classification),
            "description": self.site.config['TAG_DESCRIPTIONS'].get(lang, {}).get(classification),
            "pagekind": ["tag_page", "index" if self.show_list_as_index else "list"],
            "tag": classification,
        }
        kw.update(context)
        return context, kw

    def get_other_language_variants(self, classification, lang, classifications_per_language):
        """Return a list of variants of the same tag in other languages."""
        return self.translation_manager.get_translations_as_list(classification, lang, classifications_per_language)

    def postprocess_posts_per_classification(self, posts_per_classification_per_language, flat_hierarchy_per_lang=None, hierarchy_lookup_per_lang=None):
        """Rearrange, modify or otherwise use the list of posts per classification and per language."""
        self.translation_manager.read_from_config(self.site, 'TAG', posts_per_classification_per_language, False)



# -*- coding: utf-8 -*-
#
# Copyright © 2013-2018 Michael Rabbitt, Roberto Alsina and others.
#
# Permission is hereby granted, free of charge, to any person obtaining a
# copy of this software and associated documentation files (the
# "Software"), to deal in the Software without restriction, including
# without limitation the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the Software, and to
# permit persons to whom the Software is furnished to do so, subject to
# the following conditions:
#
# The above copyright notice and this permission notice shall be included
# in all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS
# OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
# MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.
# IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY
# CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
#
# Inspired by "[Python] reStructuredText GitHub Podcast directive"
# (https://gist.github.com/brianhsu/1407759), public domain by Brian Hsu

"""
Extension to Python Markdown for Embedded Audio.

Basic Example:

>>> import markdown
>>> text = "[podcast]https://archive.org/download/Rebeldes_Stereotipos/rs20120609_1.mp3[/podcast]"
>>> html = markdown.markdown(text, [PodcastExtension()])
>>> print(html)
<p><audio controls=""><source src="https://archive.org/download/Rebeldes_Stereotipos/rs20120609_1.mp3" type="audio/mpeg"></source></audio></p>
"""

from nikola.plugin_categories import MarkdownExtension
try:
    from markdown.extensions import Extension
    from markdown.inlinepatterns import Pattern
    from markdown.util import etree
except ImportError:
    # No need to catch this, if you try to use this without Markdown,
    # the markdown compiler will fail first
    Pattern = Extension = object

PODCAST_RE = r'\[podcast\](?P<url>.+)\[/podcast\]'


class PodcastPattern(Pattern):
    """InlinePattern for footnote markers in a document's body text."""

    def __init__(self, pattern, configs):
        """Initialize pattern."""
        Pattern.__init__(self, pattern)

    def handleMatch(self, m):
        """Handle pattern matches."""
        url = m.group('url').strip()
        audio_elem = etree.Element('audio')
        audio_elem.set('controls', '')
        source_elem = etree.SubElement(audio_elem, 'source')
        source_elem.set('src', url)
        source_elem.set('type', 'audio/mpeg')
        return audio_elem


class PodcastExtension(MarkdownExtension, Extension):
    """Podcast extension for Markdown."""

    def __init__(self, configs={}):
        """Initialize extension."""
        # set extension defaults
        self.config = {}

        # Override defaults with user settings
        for key, value in configs:
            self.setConfig(key, value)

    def extendMarkdown(self, md, md_globals):
        """Extend Markdown."""
        podcast_md_pattern = PodcastPattern(PODCAST_RE, self.getConfigs())
        podcast_md_pattern.md = md
        md.inlinePatterns.add('podcast', podcast_md_pattern, "<not_strong")
        md.registerExtension(self)


def makeExtension(configs=None):  # pragma: no cover
    """Make Markdown extension."""
    return PodcastExtension(configs)


if __name__ == '__main__':
    import doctest
    doctest.testmod(optionflags=(doctest.NORMALIZE_WHITESPACE +
                                 doctest.REPORT_NDIFF))



#!/usr/bin/env python3

from nikola import nikola
n = nikola.Nikola()
n.init_plugins()

print(""".. title: Path Handlers for Nikola
.. slug: path-handlers
.. author: The Nikola Team

.. DO NOT EDIT, this file is auto-generated by scripts/document_path_handlers.py

Nikola supports special links with the syntax ``link://kind/name``. In
templates you can also use ``_link(kind, name)``.  You can add query strings
(``?key=value``) for extra arguments, or pass keyword arguments to ``_link`` in
templates (support and behavior depends on path handlers themselves). Fragments
(``#anchor``) will be appended to the transformed link.

Here are the descriptions for all the supported kinds.

.. class:: dl-horizontal
""")

for k in sorted(n.path_handlers.keys()):
    v = n.path_handlers[k]
    print(k)
    print('\n'.join('    ' + l.strip() for l in v.__doc__.splitlines()))
    print()



# -*- coding: utf-8 -*-

# Copyright © 2012-2018 Roberto Alsina and others.

# Permission is hereby granted, free of charge, to any
# person obtaining a copy of this software and associated
# documentation files (the "Software"), to deal in the
# Software without restriction, including without limitation
# the rights to use, copy, modify, merge, publish,
# distribute, sublicense, and/or sell copies of the
# Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice
# shall be included in all copies or substantial portions of
# the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY
# KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
# WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR
# PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS
# OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR
# OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR
# OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
# SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

"""Page compiler plugin for pandoc.

You will need, of course, to install pandoc
"""


import io
import os
import subprocess

from nikola.plugin_categories import PageCompiler
from nikola.utils import req_missing, makedirs, write_metadata


class CompilePandoc(PageCompiler):
    """Compile markups into HTML using pandoc."""

    name = "pandoc"
    friendly_name = "pandoc"

    def set_site(self, site):
        """Set Nikola site."""
        self.config_dependencies = [str(site.config['PANDOC_OPTIONS'])]
        super(CompilePandoc, self).set_site(site)

    def compile(self, source, dest, is_two_file=True, post=None, lang=None):
        """Compile the source file into HTML and save as dest."""
        makedirs(os.path.dirname(dest))
        try:
            subprocess.check_call(['pandoc', '-o', dest, source] + self.site.config['PANDOC_OPTIONS'])
            with open(dest, 'r', encoding='utf-8') as inf:
                output, shortcode_deps = self.site.apply_shortcodes(inf.read(), with_dependencies=True)
            with open(dest, 'w', encoding='utf-8') as outf:
                outf.write(output)
            if post is None:
                if shortcode_deps:
                    self.logger.error(
                        "Cannot save dependencies for post {0} (post unknown)",
                        source)
            else:
                post._depfile[dest] += shortcode_deps
        except OSError as e:
            if e.strreror == 'No such file or directory':
                req_missing(['pandoc'], 'build this site (compile with pandoc)', python=False)

    def compile_string(self, data, source_path=None, is_two_file=True, post=None, lang=None):
        """Compile into HTML strings."""
        raise ValueError("Pandoc compiler does not support compile_string due to multiple output formats")

    def create_post(self, path, **kw):
        """Create a new post."""
        content = kw.pop('content', None)
        onefile = kw.pop('onefile', False)
        # is_page is not used by create_post as of now.
        kw.pop('is_page', False)
        metadata = {}
        metadata.update(self.default_metadata)
        metadata.update(kw)
        makedirs(os.path.dirname(path))
        if not content.endswith('\n'):
            content += '\n'
        with io.open(path, "w+", encoding="utf8") as fd:
            if onefile:
                fd.write(write_metadata(metadata, comment_wrap=True, site=self.site, compiler=self))
            fd.write(content)



# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "पढ़ने में %d मिनट बाकी",
    "(active)": "(सक्रिय)",
    "Also available in:": "उपलब्ध भाषाएँ:",
    "Archive": "आर्काइव",
    "Atom feed": "",
    "Authors": "लेखक",
    "Categories": "श्रेणियाँ",
    "Comments": "टिप्पणियाँ",
    "LANGUAGE": "हिन्दी",
    "Languages:": "भाषाएँ:",
    "More posts about %s": "%s के बारे में अौर पोस्टें",
    "Newer posts": "नई पोस्टें",
    "Next post": "अगली पोस्ट",
    "Next": "",
    "No posts found.": "कोई पोस्ट नहीं मिल सकी",
    "Nothing found.": "कुछ नहीं मिल सका",
    "Older posts": "पुरानी पोस्टें",
    "Original site": "असली साइट",
    "Posted:": "पोस्टेड:",
    "Posts about %s": "%s के बारे में पोस्टें",
    "Posts by %s": "%s  की पोस्टें",
    "Posts for year %s": "साल %s की पोस्टें",
    "Posts for {month_day_year}": "{month_day_year} की पोस्टें",
    "Posts for {month_year}": "{month_year} की पोस्टें",
    "Previous post": "पिछली पोस्ट",
    "Previous": "",
    "Publication date": "प्रकाशन की तारीख",
    "RSS feed": "आर एस एस फ़ीड",
    "Read in English": "हिन्दी में पढ़िए",
    "Read more": "और पढ़िए",
    "Skip to main content": "मुख्य सामग्री पर जाएँ",
    "Source": "सोर्स",
    "Subcategories:": "उपश्रेणी",
    "Tags and Categories": "टैग्स और श्रेणियाँ",
    "Tags": "टैग्स",
    "Toggle navigation": "",
    "Uncategorized": "बिना श्रेणी",
    "Up": "",
    "Updates": "अपडेट्स",
    "Write your page here.": "अपना पेज यहाँ लिखिए",
    "Write your post here.": "अपनी पोस्ट यहाँ लिखिए",
    "old posts, page %d": "पुरानी पोस्टें, पृष्‍ठ %d",
    "page %d": "पृष्‍ठ %d",
    "updated": "",
}



# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "残りを読むのに必要な時間は%d分",
    "(active)": "(有効)",
    "Also available in:": "他の言語で読む:",
    "Archive": "文書一覧",
    "Atom feed": "Atomフィード",
    "Authors": "著者一覧",
    "Categories": "カテゴリ",
    "Comments": "コメント",
    "LANGUAGE": "日本語",
    "Languages:": "言語:",
    "More posts about %s": "%sに関する文書一覧",
    "Newer posts": "新しい文書",
    "Next post": "次の文書",
    "Next": "次",
    "No posts found.": "文書はありません。",
    "Nothing found.": "なにも見つかりませんでした。",
    "Older posts": "過去の文書",
    "Original site": "翻訳元のサイト",
    "Posted:": "公開日時:",
    "Posts about %s": "%sについての文書",
    "Posts by %s": "%sの文書一覧",
    "Posts for year %s": "%s年の文書",
    "Posts for {month_day_year}": "{month_day_year}の文書",
    "Posts for {month_year}": "{month_year}の文書",
    "Previous post": "一つ前の文書",
    "Previous": "前",
    "Publication date": "公開日",
    "RSS feed": "RSSフィード",
    "Read in English": "日本語で読む",
    "Read more": "続きを読む",
    "Skip to main content": "本文を読み飛ばす",
    "Source": "ソース",
    "Subcategories:": "サブカテゴリ",
    "Tags and Categories": "カテゴリおよびタグ一覧",
    "Tags": "タグ",
    "Toggle navigation": "ナビゲーションを隠す",
    "Uncategorized": "uncategorized",
    "Up": "上",
    "Updates": "フィード",
    "Write your page here.": "ここに文書を記述してください。",
    "Write your post here.": "ここに文書を記述してください。",
    "old posts, page %d": "過去の文書 %dページ目",
    "page %d": "ページ%d",
    "updated": "",
}



# -*- encoding:utf-8 -*-
"""Autogenerated file, do not edit. Submit translations on Transifex."""

MESSAGES = {
    "%d min remaining to read": "%d min ngelen për tu lexuar",
    "(active)": "(aktiv)",
    "Also available in:": "Gjithashtu e disponueshme në:",
    "Archive": "Arkiva",
    "Atom feed": "",
    "Authors": "Autorë",
    "Categories": "Kategori",
    "Comments": "Komente",
    "LANGUAGE": "Shqip",
    "Languages:": "Gjuhë:",
    "More posts about %s": "Më shumë postime rreth %s",
    "Newer posts": "Postime më të reja",
    "Next post": "Postimi i rradhës",
    "Next": "",
    "No posts found.": "Nuk është gjetur asnjë post.",
    "Nothing found.": "Nuk është gjetur asgjë.",
    "Older posts": "Postime më të vjetra",
    "Original site": "Faqja origjinale",
    "Posted:": "Postuar:",
    "Posts about %s": "Postime rreth %s",
    "Posts by %s": "Postime nga %s",
    "Posts for year %s": "Postime për vitin %s",
    "Posts for {month_day_year}": "Postime për {month_day_year}",
    "Posts for {month_year}": "Postime për {month_year}",
    "Previous post": "Postim i kaluar",
    "Previous": "",
    "Publication date": "Data e publikimit",
    "RSS feed": "Furnizim RSS",
    "Read in English": "Lexo në Shqip",
    "Read more": "Lexo më shumë",
    "Skip to main content": "Shko tek përmbajtja kryesore",
    "Source": "Burim",
    "Subcategories:": "Nënkategori:",
    "Tags and Categories": "Etiketa dhe Kategori",
    "Tags": "Etiketa",
    "Toggle navigation": "",
    "Uncategorized": "E pa kategorizuar",
    "Up": "",
    "Updates": "Përditësime",
    "Write your page here.": "Shkruaj faqen tënde këtu.",
    "Write your post here.": "Shkruaj postin tënd këtu.",
    "old posts, page %d": "Postime të kaluara, faqe %d",
    "page %d": "faqe %d",
    "updated": "",
}



# -*- coding: utf-8 -*-
# As prescribed in README.rst:
import os
import unittest
import tempfile
from PIL import Image, ImageDraw

from nikola.plugins.task import scale_images
# Import test - should perhaps be moved to a separate module
import nikola.plugins.task.galleries  # NOQA
from .base import FakeSite


class TestCase(unittest.TestCase):
    def setUp(self):
        # These tests don't require valid profiles.  They need only to verify
        # that profile data is/isn't saved with images.
        # It would be nice to use PIL.ImageCms to create valid profiles, but
        # in many Pillow distributions ImageCms is a stub.
        # ICC file data format specification:
        # http://www.color.org/icc32.pdf

        self._profile = b'invalid profile data'

        # Make a white image with a red stripe on the diagonal.
        w = 64
        h = 64
        img = Image.new("RGB", (w, h), (255, 255, 255))
        draw = ImageDraw.Draw(img)
        draw.line((0, 0, w, h), fill=(255, 128, 128))
        draw.line((w, 0, 0, h), fill=(128, 128, 255))
        self._img = img

        self._src_dir = tempfile.TemporaryDirectory()
        self._dest_dir = tempfile.TemporaryDirectory()

    def tearDown(self):
        pass

    def _tmp_img_name(self, dirname):
        pathname = tempfile.NamedTemporaryFile(
            suffix=".jpg", dir=dirname, delete=False)
        return pathname.name

    def _get_site(self, preserve_icc_profiles):
        site = FakeSite()
        site.config['IMAGE_FOLDERS'] = {self._src_dir.name: ''}
        site.config['OUTPUT_FOLDER'] = self._dest_dir.name
        site.config['IMAGE_THUMBNAIL_SIZE'] = 128
        site.config['IMAGE_THUMBNAIL_FORMAT'] = '{name}.thumbnail{ext}'
        site.config['MAX_IMAGE_SIZE'] = 512
        site.config['FILTERS'] = {}
        site.config['PRESERVE_EXIF_DATA'] = False
        site.config['EXIF_WHITELIST'] = {}
        site.config['PRESERVE_ICC_PROFILES'] = preserve_icc_profiles
        return site

    def _get_task_instance(self, preserve_icc_profiles):
        result = scale_images.ScaleImage()
        result.set_site(self._get_site(preserve_icc_profiles))
        return result

    def _create_src_images(self):
        img = self._img
        # Test two variants: with and without an associated icc_profile
        pathname = self._tmp_img_name(self._src_dir.name)
        img.save(pathname)
        sans_icc_filename = os.path.basename(pathname)

        pathname = self._tmp_img_name(self._src_dir.name)
        img.save(pathname, icc_profile=self._profile)
        with_icc_filename = os.path.basename(pathname)
        return [sans_icc_filename, with_icc_filename]

    def _run_task(self, preserve_icc_profiles):
        task_instance = self._get_task_instance(preserve_icc_profiles)
        for task in task_instance.gen_tasks():
            for action, args in task.get('actions', []):
                action(*args)

    def test_scale_preserving_icc_profile(self):
        sans_icc_filename, with_icc_filename = self._create_src_images()
        self._run_task(True)
        cases = [
            (sans_icc_filename, None),
            (with_icc_filename, self._profile),
        ]
        for (filename, expected_profile) in cases:
            pathname = os.path.join(self._dest_dir.name, filename)
            self.assertTrue(os.path.exists(pathname), pathname)
            img = Image.open(pathname)
            actual_profile = img.info.get('icc_profile')
            self.assertEqual(actual_profile, expected_profile)

    def test_scale_discarding_icc_profile(self):
        sans_icc_filename, with_icc_filename = self._create_src_images()
        self._run_task(False)
        cases = [
            (sans_icc_filename, None),
            (with_icc_filename, None),
        ]
        for (filename, expected_profile) in cases:
            pathname = os.path.join(self._dest_dir.name, filename)
            self.assertTrue(os.path.exists(pathname), pathname)
            img = Image.open(pathname)
            actual_profile = img.info.get('icc_profile')
            self.assertEqual(actual_profile, expected_profile)


main = unittest.main

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# -*- encoding: utf-8 -*-
# Date Conditionals v0.1.6
# Copyright © 2015-2018, Chris Warrick.
# All rights reserved.
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. Neither the name of the author of this software nor the names of
#    contributors to this software may be used to endorse or promote
#    products derived from this software without specific prior written
#    consent.
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
# LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
# A PARTICULAR PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT
# OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
# LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
# DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
# THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
# (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
# OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

"""Date range parser."""

from __future__ import print_function, unicode_literals
import datetime
import dateutil.parser
import re
import operator


__all__ = ('date_in_range',)
CLAUSE = re.compile('(year|month|day|hour|minute|second|weekday|isoweekday)?'
                    ' ?(==|!=|<=|>=|<|>) ?(.*)')
OPERATORS = {
    '==': operator.eq,
    '!=': operator.ne,
    '<=': operator.le,
    '>=': operator.ge,
    '<': operator.lt,
    '>': operator.gt,
}


def date_in_range(date_range, date, debug=False, now=None):
    """Check if date is in the range specified.

    Format:
    * comma-separated clauses (AND)
    * clause: attribute comparison_operator value (spaces optional)
        * attribute: year, month, day, hour, month, second, weekday, isoweekday
          or empty for full datetime
        * comparison_operator: == != <= >= < >
        * value: integer, 'now' or dateutil-compatible date input

    The optional `now` parameter can be used to provide a specific `now` value
    (if none is provided, datetime.datetime.now() is used).
    """
    out = True

    for item in date_range.split(','):
        attribute, comparison_operator, value = CLAUSE.match(
            item.strip()).groups()
        if attribute in ('weekday', 'isoweekday'):
            left = getattr(date, attribute)()
            right = int(value)
        elif attribute:
            left = getattr(date, attribute)
            right = int(value)
        elif value == 'now':
            left = date
            right = now or datetime.datetime.now()
        else:
            left = date
            right = dateutil.parser.parse(value)
        if debug:  # pragma: no cover
            print("    <{0} {1} {2}>".format(left, comparison_operator, right))
        out = out and OPERATORS[comparison_operator](left, right)
    return out
